@@ -524,7 +524,6 @@ sub _serve_request_multiple {
 
     @multiple_files = split(/,/, $list);
 
-    # TODO: don't allow this if multiple files aren't enabled
     return $self->_simple_response(403, "Multiple file serving isn't enabled") unless $svc->{enable_concatenate_get};
     return $self->_simple_response(403, "Too many files requested") if @multiple_files > 100;
 
@@ -610,8 +609,7 @@ sub _serve_request_multiple_poststat {
 
     # gotta send all files, one by one...
     my @remain = @$filelist;
-    my $do_next;
-    $do_next = sub {
+    $self->{post_sendfile_cb} = sub {
         unless (@remain) {
             $self->write(sub { $self->http_response_sent; });
             return;
@@ -638,11 +636,10 @@ sub _serve_request_multiple_poststat {
             }
 
             $self->{reproxy_file}     = $file;
-            $self->{post_sendfile_cb} = $do_next;
             $self->reproxy_fh($rp_fh, $size);
         });
     };
-    $do_next->();
+    $self->{post_sendfile_cb}->();
 }
 
 sub try_index_files {
